//
//  VTSearcher.m
//  VecTripper
//
//  Created by tmkk on Mon Mar 07 2005.
//  Copyright (c) 2005 tmkk. All rights reserved.
//

#import "VTSearcher.h"
#import "VTController.h"

#import <pcre.h>
//#import <oniguruma.h>
#import <sys/time.h>
#import "arch.h"
#import "DES_bs.h"
//#import "ac.h"
//#import "aho-corasick.h"

//#define HASH_SIZE 4082747
//#define HASH_SIZE 4194304
//#define HASH_SIZE 4159647
//#define HASH_SIZE_HALF 2079823

/* Period parameters */  
#define N 624
#define M 397
#define MATRIX_A 0x9908b0dfUL   /* constant vector a */
#define UMASK 0x80000000UL /* most significant w-r bits */
#define LMASK 0x7fffffffUL /* least significant r bits */
#define MIXBITS(u,v) ( ((u) & UMASK) | ((v) & LMASK) )
#define TWIST(u,v) ((MIXBITS(u,v) >> 1) ^ ((v)&1UL ? MATRIX_A : 0UL))

static unsigned long state[N]; /* the array for the state vector  */
static int left = 1;
static int initf = 0;
static unsigned long *next;

/* initializes state[N] with a seed */
void init_genrand(unsigned long s)
{
    int j;
    state[0]= s & 0xffffffffUL;
    for (j=1; j<N; j++) {
        state[j] = (1812433253UL * (state[j-1] ^ (state[j-1] >> 30)) + j); 
        /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
        /* In the previous versions, MSBs of the seed affect   */
        /* only MSBs of the array state[].                        */
        /* 2002/01/09 modified by Makoto Matsumoto             */
        state[j] &= 0xffffffffUL;  /* for >32 bit machines */
    }
    left = 1; initf = 1;
}

static void next_state(void)
{
    unsigned long *p=state;
    int j;

    /* if init_genrand() has not been called, */
    /* a default initial seed is used         */
    if (initf==0) init_genrand(5489UL);

    left = N;
    next = state;
    
    for (j=N-M+1; --j; p++) 
        *p = p[M] ^ TWIST(p[0], p[1]);

    for (j=M; --j; p++) 
        *p = p[M-N] ^ TWIST(p[0], p[1]);

    *p = p[M-N] ^ TWIST(p[0], state[0]);
}

unsigned long genrand_int32(void)
{
    unsigned long y;

    if (--left == 0) next_state();
    y = *next++;

    /* Tempering */
    y ^= (y >> 11);
    y ^= (y << 7) & 0x9d2c5680UL;
    y ^= (y << 15) & 0xefc60000UL;
    y ^= (y >> 18);

    return y;
}

extern char itoa64[64];

struct search_method
{
	int (*gettrip)(unsigned char *rawtrip);
	int (*match)(pcre *re, char *trip, int cont, int min);
};

static pcre_extra *pe;

static unsigned char DES_atoi64[0x100] = {
	18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
	34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
	50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 0, 1,
	2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 5, 6, 7, 8, 9, 10,
	11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26,
	27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 32, 33, 34, 35, 36,
	37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52,
	53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 0, 1, 2, 3, 4,
	5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
	21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36,
	37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52,
	53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 0, 1, 2, 3, 4,
	5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
	21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36,
	37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52,
	53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 0, 1, 2, 3, 4
};

static unsigned char DES_FP[64] = {
	39, 7,47,15,55,23,63,31,
	38, 6,46,14,54,22,62,30,
	37, 5,45,13,53,21,61,29,
	36, 4,44,12,52,20,60,28,
	35, 3,43,11,51,19,59,27,
	34, 2,42,10,50,18,58,26,
	33, 1,41, 9,49,17,57,25,
	32, 0,40, 8,48,16,56,24
};

static unsigned char DES_IP[64] = {
	57, 49, 41, 33, 25, 17, 9, 1,
	59, 51, 43, 35, 27, 19, 11, 3,
	61, 53, 45, 37, 29, 21, 13, 5,
	63, 55, 47, 39, 31, 23, 15, 7,
	56, 48, 40, 32, 24, 16, 8, 0,
	58, 50, 42, 34, 26, 18, 10, 2,
	60, 52, 44, 36, 28, 20, 12, 4,
	62, 54, 46, 38, 30, 22, 14, 6
};

/*static unsigned char nextchar[128] = {
	33, 33, 33, 33, 33, 33, 33, 33,
	33, 33, 33, 33, 33, 33, 33, 33,
	33, 33, 33, 33, 33, 33, 33, 33,
	33, 33, 33, 33, 33, 33, 33, 33,
	33, 34, 35, 36, 37, 38, 39, 40,
	41, 42, 43, 44, 45, 46, 47, 48,
	49, 50, 51, 52, 53, 54, 55, 56,
	57, 58, 59, 60, 61, 62, 63, 64,
	65, 66, 67, 68, 69, 70, 71, 72,
	73, 74, 75, 76, 77, 78, 79, 80,
	81, 82, 83, 84, 85, 86, 87, 88,
	89, 90, 91, 92, 93, 94, 95, 96,
	97, 98, 99, 100,101,102,103,104,
	105,106,107,108,109,110,111,112,
	113,114,115,116,117,118,119,120,
	121,122,123,124,125,126,33, 33
};*/

static unsigned int __attribute__ ((aligned (16))) masktable[128];
static int shifttable[64];
//static unsigned int ofstable[128][2];
static ARCH_WORD *pat_arr;
static ARCH_WORD *msk_arr;
//static unsigned int *msk_arr_high;
//static unsigned int *msk_arr_low;
//static ARCH_WORD *pat_arr_old;
//static ARCH_WORD *msk_arr_old;

void john_crypt_setsalt(char *salt)
{
	ARCH_WORD binsalt = DES_atoi64[salt[0]] | ((ARCH_WORD)DES_atoi64[salt[1]] << 6);
	DES_bs_set_salt(binsalt);
}


void generate_shifttable()
{
	int i,j=2;
	for(i=6;i<64;i++) {
		shifttable[i] = 1 << (j*6 - 1 - i);
		if(i == 6*j-1) j++;
	}
}

void generate_masktable()
{
	int i;
	for(i=0;i<128;i++) {
		masktable[i] = 1 << (i & 0x1f);
	}
}

void generate_pattern(char *pattern, ARCH_WORD *pattern_arr)
{
	ARCH_WORD block[3];
	ARCH_WORD mask;
	int chr, src, dst, value;

	block[0] = block[1] = 0;
	dst = 0;
	for (chr = 0; chr < 11; chr++) {
		value = DES_atoi64[ARCH_INDEX(pattern[chr])];
		mask = 0x20;

		for (src = 0; src < 6; src++) {
			if (value & mask)
				block[dst >> 5] |= 1 << (dst & 0x1F);
			mask >>= 1;
			dst++;
		}
	}
	
	pattern_arr[0] = pattern_arr[1] = 0;
	for (dst = 0; dst < 64; dst++) {
		src = DES_IP[dst];

		if (block[src >> 5] & (1 << (src & 0x1F)))
			pattern_arr[dst >> 5] |= 1 << (dst & 0x1F);
	}
}

void generate_mask(char *pattern, ARCH_WORD *mask_arr)
{
	ARCH_WORD block[3];
	ARCH_WORD mask;
	int chr, src, dst, value;

	block[0] = block[1] = 0;
	dst = 0;
	for (chr = 0; chr < 11; chr++) {
		if(pattern[chr] == '?') value = 0;
		else value = 0x3f;
		mask = 0x20;

		for (src = 0; src < 6; src++) {
			if (value & mask)
				block[dst >> 5] |= 1 << (dst & 0x1F);
			mask >>= 1;
			dst++;
		}
	}
	
	mask_arr[0] = mask_arr[1] = 0;
	for (dst = 0; dst < 64; dst++) {
		src = DES_IP[dst];

		if (block[src >> 5] & (1 << (src & 0x1F)))
			mask_arr[dst >> 5] |= 1 << (dst & 0x1F);
	}
}

void generate_pattern2(char *pattern, ARCH_WORD *pattern_arr)
{
	ARCH_WORD block[3];
	ARCH_WORD mask;
	int chr, src, dst, value;

	block[0] = block[1] = 0;
	dst = 0;
	for (chr = 0; chr < 11; chr++) {
		value = DES_atoi64[ARCH_INDEX(pattern[chr])];
		mask = 0x20;

		for (src = 0; src < 6; src++) {
			if (value & mask)
				block[dst >> 5] |= 1 << (dst & 0x1F);
			mask >>= 1;
			dst++;
		}
	}
	
	pattern_arr[0] = pattern_arr[4] = 0;
	for (dst = 0; dst < 32; dst++) {
		src = DES_IP[dst];

		if (block[src >> 5] & (1 << (src & 0x1F)))
			pattern_arr[0] |= 1 << (dst & 0x1F);
	}
	for (dst = 32; dst < 64; dst++) {
		src = DES_IP[dst];

		if (block[src >> 5] & (1 << (src & 0x1F)))
			pattern_arr[4] |= 1 << (dst & 0x1F);
	}
}

void generate_mask2(char *pattern, ARCH_WORD *mask_arr)
{
	ARCH_WORD block[3];
	ARCH_WORD mask;
	int chr, src, dst, value;

	block[0] = block[1] = 0;
	dst = 0;
	for (chr = 0; chr < 11; chr++) {
		if(pattern[chr] == '?') value = 0;
		else value = 0x3f;
		mask = 0x20;

		for (src = 0; src < 6; src++) {
			if (value & mask)
				block[dst >> 5] |= 1 << (dst & 0x1F);
			mask >>= 1;
			dst++;
		}
	}
	
	mask_arr[0] = mask_arr[4] = 0;
	for (dst = 0; dst < 32; dst++) {
		src = DES_IP[dst];

		if (block[src >> 5] & (1 << (src & 0x1F)))
			mask_arr[0] |= 1 << (dst & 0x1F);
	}
	for (dst = 32; dst < 64; dst++) {
		src = DES_IP[dst];

		if (block[src >> 5] & (1 << (src & 0x1F)))
			mask_arr[4] |= 1 << (dst & 0x1F);
	}
}

int john_crypt_gettrip2(unsigned char *rawtrip)
{
	int j,cont=1,cont_max=1;
	for(j=1;j<10;j++) {
		if(!(rawtrip[j]^rawtrip[j-1])) cont++;
		else {
			if(cont > cont_max) cont_max = cont;
			cont = 1;
		}
	}
	if(cont > cont_max) cont_max = cont;
	
	return cont_max;
}

int john_crypt_gettrip3(unsigned char *rawtrip)
{
	int i,j,cont=1,cont_max=1;
	for(j=1;j<10;j++) {
		i = rawtrip[j]^rawtrip[j-1];
		if(!i || i == 0x20) cont++;
		else {
			if(cont > cont_max) cont_max = cont;
			cont = 1;
		}
	}
	if(cont > cont_max) cont_max = cont;
	
	return cont_max;
}

/*
inline int john_crypt_gettrip(char *trip, unsigned char *rawtrip1, unsigned char *rawtrip2, unsigned char *rawtrip3)
{
	int j;
	for(j=0;j<4;j++) {
		trip[j] = itoa64[rawtrip1[j]];
	}
	for(;j<8;j++) {
		trip[j] = itoa64[rawtrip2[j-4]];
	}
	trip[8] = itoa64[rawtrip3[0]];
	trip[9] = itoa64[rawtrip3[1]];
	return 0;
}

int john_crypt_gettrip2(char *trip, unsigned char *rawtrip1, unsigned char *rawtrip2, unsigned char *rawtrip3)
{
	int j,cont=1,cont_max=1;
	for(j=0;j<4;j++) {
		trip[j] = itoa64[rawtrip1[j]];
		if(!(trip[j]^trip[j-1])) cont++;
		else {
			if(cont > cont_max) cont_max = cont;
			cont = 1;
		}
	}
	for(;j<8;j++) {
		trip[j] = itoa64[rawtrip2[j-4]];
		if(!(trip[j]^trip[j-1])) cont++;
		else {
			if(cont > cont_max) cont_max = cont;
			cont = 1;
		}
	}
	trip[8] = itoa64[rawtrip3[0]];
	if(!(trip[8]^trip[7])) cont++;
	else {
		if(cont > cont_max) cont_max = cont;
		cont = 1;
	}
	trip[9] = itoa64[rawtrip3[1]];
	if(!(trip[9]^trip[8])) cont++;
	if(cont > cont_max) cont_max = cont;
	
	return cont_max;
}

int john_crypt_gettrip3(char *trip, unsigned char *rawtrip1, unsigned char *rawtrip2, unsigned char *rawtrip3)
{
	int i,j,cont=1,cont_max=1;
	for(j=0;j<4;j++) {
		trip[j] = itoa64[rawtrip1[j]];
		i = trip[j]^trip[j-1];
		if(!i || i == 0x20) cont++;
		else {
			if(cont > cont_max) cont_max = cont;
			cont = 1;
		}
	}
	for(;j<8;j++) {
		trip[j] = itoa64[rawtrip2[j-4]];
		i = trip[j]^trip[j-1];
		if(!i || i == 0x20) cont++;
		else {
			if(cont > cont_max) cont_max = cont;
			cont = 1;
		}
	}
	trip[8] = itoa64[rawtrip3[0]];
	i = trip[8]^trip[7];
	if(!i || i == 0x20) cont++;
	else {
		if(cont > cont_max) cont_max = cont;
		cont = 1;
	}
	trip[9] = itoa64[rawtrip3[1]];
	i = trip[9]^trip[8];
	if(!i || i == 0x20) cont++;
	if(cont > cont_max) cont_max = cont;
	return cont_max;
}
*/


/* for patternfile match */
int john_crypt_gettrip4(char *trip, int index, unsigned int mask)
{
	int dst, out=0, i=2;
	int idx = index >> 5;
	for (dst = 6; dst < 64; dst++) {
		if(DES_bs_all.B[DES_FP[dst]][idx] & mask) {
			out |=  shifttable[dst];
		}
		if(dst == 6*i-1) {
			trip[i-2] = itoa64[out];
			i++;
			out = 0;
		}
	}
	trip[i-2] = itoa64[out];
	
	return 1;
}

inline int john_crypt_match(pcre *re, char *trip, int cont, int min)
{
	int ovector[30];
	return (pcre_exec(re,pe,trip,10,0,0,ovector,30) > 0);
}

inline int john_crypt_match2(pcre *re, char *trip, int cont, int min)
{
	int ovector[30];
	return (pcre_exec(re,pe,trip,10,0,0,ovector,30) > 0 || cont >= min);
}

inline int john_crypt_match3(pcre *re, char *trip, int cont, int min)
{
	return (cont >= min);
}

inline int john_crypt_match4(pcre *re, char *trip, int cont, int min)
{
	return cont;
}


    void Start_or_Stop_Trace( long *result )
    {

        register long temp;

        //Generate inline assembly to read from the
        //Processor Information Register
        #if defined( __MWERKS__ )
        asm{ mfspr temp, 1023 }
        #else
        __asm__ volatile ( "mfspr %0, 1023" : "=r" (temp) );
        #endif

        //We take an address as an argument to prevent
        //the compiler from moving the code around
        *result = temp;

    }

typedef struct hash_table {
	unsigned int upper;
	unsigned int lower;
	struct hash_table *next;
} hashtable;


#define expand_bitsliced_char() \
	/* char 1, 5 */\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[63]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[60]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[31]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[28]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[38]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[35]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[6]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[3]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[46]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[43]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[14]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[11]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_three);\
	/* char 2, 6 */\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[54]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[51]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[22]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[19]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[62]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[59]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[30]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[27]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[37]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[34]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[5]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[2]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_three);\
	/* char 3, 7 */\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[45]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[42]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[13]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[10]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[53]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[50]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[21]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[18]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[61]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[58]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[29]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[26]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_three);\
	/* char 4, 8 */\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[36]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[33]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[4]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[1]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[44]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[41]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[12]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[9]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[52]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[49]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[20]);\
	v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[17]);\
	v1 = vec_and(v_mask,v1);\
	v2 = vec_and(v_mask,v2);\
	v1 = vec_rl(v1,v3);\
	v2 = vec_rl(v2,v3);\
	v_out = vec_sel(v_out,v_max,v1);\
	v_out2 = vec_sel(v_out2,v_max,v2);\
	v3 = v_rot;\
	/* char 9 */\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[57]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[25]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[32]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[0]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[40]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[8]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_three);\
	/* char 10 */\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[48]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[16]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[56]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);\
	v3 = vec_sub(v3,v_one);\
	v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[24]);\
	v1 = vec_and(v_mask,v1);\
	v1 = vec_rl(v1,v3);\
	v_out3 = vec_sel(v_out3,v_max,v1);

#define calchash(x) \
	hash=*(x);\
	hash=hash*37+*(x+1);\
	hash=hash*37+*(x+2);\
	hash=hash*37+*(x+3);\
	hash=hash*37+*(x+4);\
	hash=hash*37+*(x+5);\
	hash=hash*37+*(x+6);\
	hash=hash*37+*(x+7);

@implementation VTSearcher

- (void)search1
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	unsigned char __attribute__ ((aligned (16))) rawtrip1[16];
	char key[9];
	char trip[12];
	//char validstr[89] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz!#$%'()*+,-=?:;@`[]^_{|}~";
	char validsalt[64] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	char *errstr;
	int counter=0, next=100000;
	struct timeval tv_prev, tv_curr;
	register int i;
	int cflags=0,erroff,j,generate_mode=93;
	pcre *re = NULL;
	//char *pTrip = trip+1;
	FILE *fp;
	
	vector unsigned int v0, v1, v2, v3, v_out, v_out2, v_out3, v_mask, v_rot;
	vector unsigned char v4, v5, v_mask1;
	register vector unsigned int v_zero;
	register vector unsigned int v_one;
	register vector unsigned int v_three;
	register vector unsigned int v_max;
	register vector unsigned char v_32;
	register vector unsigned char vec_itoa1 = (vector unsigned char)(0x2e,0x2f,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x41,0x42,0x43,0x44);
	register vector unsigned char vec_itoa2 = (vector unsigned char)(0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,0x50,0x51,0x52,0x53,0x54);
	register vector unsigned char vec_itoa3 = (vector unsigned char)(0x55,0x56,0x57,0x58,0x59,0x5a,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6a);
	register vector unsigned char vec_itoa4 = (vector unsigned char)(0x6b,0x6c,0x6d,0x6e,0x6f,0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7a);
	register vector unsigned char v_zmask = (vector unsigned char)(0xff,0xff,0, 0, 0xff,0xff,0, 0, 0xff,0xff,0, 0, 0xff,0xff,0, 0);
	register vector unsigned char v_perm1 = (vector unsigned char)(0,1,2,3,16,17,18,19,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm2 = (vector unsigned char)(0,1,2,3,4,5,6,7,16,17,18,0,0,0,0,0);
	register vector unsigned char v_perm3 = (vector unsigned char)(4,5,6,7,20,21,22,23,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm4 = (vector unsigned char)(0,1,2,3,4,5,6,7,20,21,22,0,0,0,0,0);
	register vector unsigned char v_perm5 = (vector unsigned char)(8,9,10,11,24,25,26,27,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm6 = (vector unsigned char)(0,1,2,3,4,5,6,7,24,25,26,0,0,0,0,0);
	register vector unsigned char v_perm7 = (vector unsigned char)(12,13,14,15,28,29,30,31,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm8 = (vector unsigned char)(0,1,2,3,4,5,6,7,28,29,30,0,0,0,0,0);
	
	
	v_zero = vec_splat_u32(0);
	v_one = vec_splat_u32(1);
	v_three = vec_splat_u32(3);
	v_32 = (vector unsigned char)(32);
	v_max = vec_splat_u32(0xffffffff);
	
	pe = NULL;
	trip[0] = -1;
	trip[11] = 0;
	key[8] = 0;
	
	if(ignoreCase) cflags |= PCRE_CASELESS;
	re = pcre_compile([searchStr UTF8String], cflags, &errstr, &erroff, NULL);
	if(re == NULL) {
		[delegate error: errstr type:0];
		[pool release];
		return;
	}
	if(study) pe = pcre_study(re, 0, &errstr);
	
	if(shouldLog) {
		fp = fopen(logPath, "ab");
		if(!fp) {
			[delegate error: nil type:1];
			if(re) free(re);
			if(pe) free(pe);
			[pool release];
			return;
		}
	}
		
	*(key+1) = validsalt[genrand_int32()%64];
	*(key+2) = validsalt[genrand_int32()%64];
	john_crypt_setsalt(key+1);
	
	//stime = time(NULL);
	gettimeofday(&tv_prev, NULL);
	
	while(running) {
		if(generate_mode < 93) {
			for(i=0;i<128;i++) {
				memcpy(key,DES_bs_all.keys[i],8);
				key[0] = (key[0] < 126) ? key[0]+1 : 33;
				DES_bs_set_key(key, i);
			}
			generate_mode++;
		}
		else {
			for(i=0;i<128;i++) {
				key[0] = genrand_int32()%94+33;
				key[3] = genrand_int32()%94+33;
				key[4] = genrand_int32()%94+33;
				key[5] = genrand_int32()%94+33;
				key[6] = genrand_int32()%94+33;
				key[7] = genrand_int32()%94+33;
				DES_bs_set_key(key, i);
			}
			generate_mode = 0;
		}
		
		DES_bs_expand_keys();
		DES_bs_crypt_25();
		
		v_mask = vec_splat_u32(1);
		v_rot = (vector unsigned int)(61);
		
		for(i=0;i<32;i++) {
			v3 = v_rot;
			v_out = v_zero;
			v_out2 = v_zero;
			v_out3 = v_zero;
			
			expand_bitsliced_char()
			
			v_mask = vec_sl(v_mask, v_one);
			v_rot = vec_sub(v_rot,v_one);
			
			(vector bool char)v_mask1 = vec_cmplt((vector unsigned char)v_out, v_32);
			v4 = vec_perm(vec_itoa1, vec_itoa2, (vector unsigned char)v_out);
			v4 = vec_and(v4, v_mask1);
			v5 = vec_perm(vec_itoa3, vec_itoa4, (vector unsigned char)v_out);
			v5 = vec_andc(v5, v_mask1);
			(vector unsigned char)v_out = vec_or(v4, v5);
			
			(vector bool char)v_mask1 = vec_cmplt((vector unsigned char)v_out2, v_32);
			v4 = vec_perm(vec_itoa1, vec_itoa2, (vector unsigned char)v_out2);
			v4 = vec_and(v4, v_mask1);
			v5 = vec_perm(vec_itoa3, vec_itoa4, (vector unsigned char)v_out2);
			v5 = vec_andc(v5, v_mask1);
			(vector unsigned char)v_out2 = vec_or(v4, v5);
			
			(vector bool char)v_mask1 = vec_cmplt((vector unsigned char)v_out3, v_32);
			v4 = vec_perm(vec_itoa1, vec_itoa2, (vector unsigned char)v_out3);
			v4 = vec_and(v4, v_mask1);
			v5 = vec_perm(vec_itoa3, vec_itoa4, (vector unsigned char)v_out3);
			v5 = vec_andc(v5, v_mask1);
			(vector unsigned char)v_out3 = vec_or(v4, v5);
			(vector unsigned char)v_out3 = vec_and(v_zmask, (vector unsigned char)v_out3);
			
			v0 = vec_perm(v_out,v_out2,v_perm1);
			v0 = vec_perm(v0,v_out3,v_perm2);
			v1 = vec_perm(v_out,v_out2,v_perm3);
			v1 = vec_perm(v1,v_out3,v_perm4);
			v2 = vec_perm(v_out,v_out2,v_perm5);
			v2 = vec_perm(v2,v_out3,v_perm6);
			v3 = vec_perm(v_out,v_out2,v_perm7);
			v3 = vec_perm(v3,v_out3,v_perm8);
			
			vec_st(v0, 0, (unsigned int *)rawtrip1);
			//if(!strncmp(rawtrip1,"hogeho",6)) {
			if(john_crypt_match(re,rawtrip1,0,0)) {
				memcpy(key, DES_bs_all.keys[i], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
			
			vec_st(v1, 0, (unsigned int *)rawtrip1);
			//if(!strncmp(rawtrip1,"hogeho",6)) {
			if(john_crypt_match(re,rawtrip1,0,0)) {
				memcpy(key, DES_bs_all.keys[i+32], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
			
			vec_st(v2, 0, (unsigned int *)rawtrip1);
			//if(!strncmp(rawtrip1,"hogeho",6)) {
			if(john_crypt_match(re,rawtrip1,0,0)) {
				memcpy(key, DES_bs_all.keys[i+64], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
			
			vec_st(v3, 0, (unsigned int *)rawtrip1);
			//if(!strncmp(rawtrip1,"hogeho",6)) {
			if(john_crypt_match(re,rawtrip1,0,0)) {
				memcpy(key, DES_bs_all.keys[i+96], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
		}
		
		counter++;
		gettimeofday(&tv_curr,NULL);
		if((i = tv_curr.tv_sec-tv_prev.tv_sec) >= 5)
		{
			[delegate setCurrentSpeed:(double)(counter*128)/((i*1000000+tv_curr.tv_usec-tv_prev.tv_usec)/1000000.0)];
			tv_prev = tv_curr;
			next = next - counter;
			if(next <= 0) {
				*(key+1) = validsalt[genrand_int32()%64];
				*(key+2) = validsalt[genrand_int32()%64];
				john_crypt_setsalt(key+1);
				next = 100000;
				generate_mode = 93;
			}
			counter = 0;
		}
	}
	
	if(re) free(re);
	if(pe) free(pe);
	if(shouldLog) fclose(fp);
	[pool release];
}

- (void)search2
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	unsigned char __attribute__ ((aligned (16))) rawtrip1[16];
	//unsigned char __attribute__ ((aligned (16))) rawtrip2[16];
	//unsigned char __attribute__ ((aligned (16))) rawtrip3[16];
	char key[9];
	char trip[12];
	//char validstr[89] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz!#$%'()*+,-=?:;@`[]^_{|}~";
	char validsalt[64] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	char *errstr;
	int counter=0, next=100000;
	struct timeval tv_prev, tv_curr;
	register int i;
	int cflags=0,erroff,j,generate_mode=93,ret;
	pcre *re = NULL;
	struct search_method method;
	FILE *fp;
	
	vector unsigned int v0, v1, v2, v3, v_out, v_out2, v_out3, v_mask, v_rot;
	vector unsigned char v4, v5, v_mask1;
	register vector unsigned int v_zero;
	register vector unsigned int v_one;
	register vector unsigned int v_three;
	register vector unsigned int v_max;
	register vector unsigned char v_32;
	register vector unsigned char vec_itoa1 = (vector unsigned char)(0x2e,0x2f,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x41,0x42,0x43,0x44);
	register vector unsigned char vec_itoa2 = (vector unsigned char)(0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,0x50,0x51,0x52,0x53,0x54);
	register vector unsigned char vec_itoa3 = (vector unsigned char)(0x55,0x56,0x57,0x58,0x59,0x5a,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6a);
	register vector unsigned char vec_itoa4 = (vector unsigned char)(0x6b,0x6c,0x6d,0x6e,0x6f,0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7a);
	register vector unsigned char v_zmask = (vector unsigned char)(0xff,0xff,0, 0, 0xff,0xff,0, 0, 0xff,0xff,0, 0, 0xff,0xff,0, 0);
	register vector unsigned char v_perm1 = (vector unsigned char)(0,1,2,3,16,17,18,19,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm2 = (vector unsigned char)(0,1,2,3,4,5,6,7,16,17,18,0,0,0,0,0);
	register vector unsigned char v_perm3 = (vector unsigned char)(4,5,6,7,20,21,22,23,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm4 = (vector unsigned char)(0,1,2,3,4,5,6,7,20,21,22,0,0,0,0,0);
	register vector unsigned char v_perm5 = (vector unsigned char)(8,9,10,11,24,25,26,27,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm6 = (vector unsigned char)(0,1,2,3,4,5,6,7,24,25,26,0,0,0,0,0);
	register vector unsigned char v_perm7 = (vector unsigned char)(12,13,14,15,28,29,30,31,0,0,0,0,0,0,0,0);
	register vector unsigned char v_perm8 = (vector unsigned char)(0,1,2,3,4,5,6,7,28,29,30,0,0,0,0,0);
	
	v_zero = vec_splat_u32(0);
	v_one = vec_splat_u32(1);
	v_three = vec_splat_u32(3);
	v_32 = (vector unsigned char)(32);
	v_max = vec_splat_u32(0xffffffff);
	
	pe = NULL;
	trip[0] = -1;
	trip[11] = 0;
	key[8] = 0;
	
	switch(mode) {
		case 1:
			method.gettrip = john_crypt_gettrip2;
			method.match = john_crypt_match2;
			break;
		case 2:
			method.gettrip = john_crypt_gettrip3;
			method.match = john_crypt_match2;
			break;
		case 3:
			method.gettrip = john_crypt_gettrip2;
			method.match = john_crypt_match3;
			break;
		case 4:
			method.gettrip = john_crypt_gettrip3;
			method.match = john_crypt_match3;
			break;
		default:
			break;
	}
	
	if(mode < 3) {
		if(ignoreCase) cflags |= PCRE_CASELESS;
		re = pcre_compile([searchStr UTF8String], cflags, &errstr, &erroff, NULL);
		if(re == NULL) {
			[delegate error: errstr type:0];
			[pool release];
			return;
		}
		if(study) pe = pcre_study(re, 0, &errstr);
	}
	
	if(shouldLog) {
		fp = fopen(logPath, "ab");
		if(!fp) {
			[delegate error: nil type:1];
			if(re) free(re);
			if(pe) free(pe);
			[pool release];
			return;
		}
	}
		
	*(key+1) = validsalt[genrand_int32()%64];
	*(key+2) = validsalt[genrand_int32()%64];
	john_crypt_setsalt(key+1);
	
	//stime = time(NULL);
	gettimeofday(&tv_prev, NULL);
	
	while(running) {
		if(generate_mode < 93) {
			for(i=0;i<128;i++) {
				memcpy(key,DES_bs_all.keys[i],8);
				key[0] = (key[0] < 126) ? key[0]+1 : 33;
				DES_bs_set_key(key, i);
			}
			generate_mode++;
		}
		else {
			for(i=0;i<128;i++) {
				key[0] = genrand_int32()%94+33;
				key[3] = genrand_int32()%94+33;
				key[4] = genrand_int32()%94+33;
				key[5] = genrand_int32()%94+33;
				key[6] = genrand_int32()%94+33;
				key[7] = genrand_int32()%94+33;
				DES_bs_set_key(key, i);
			}
			generate_mode = 0;
		}
		
		DES_bs_expand_keys();
		DES_bs_crypt_25();
		
		v_mask = vec_splat_u32(1);
		v_rot = (vector unsigned int)(61);
		
		for(i=0;i<32;i++) {
			v3 = v_rot;
			v_out = v_zero;
			v_out2 = v_zero;
			v_out3 = v_zero;
			
			expand_bitsliced_char()
			
			v_mask = vec_sl(v_mask, v_one);
			v_rot = vec_sub(v_rot,v_one);
			
			(vector bool char)v_mask1 = vec_cmplt((vector unsigned char)v_out, v_32);
			v4 = vec_perm(vec_itoa1, vec_itoa2, (vector unsigned char)v_out);
			v4 = vec_and(v4, v_mask1);
			v5 = vec_perm(vec_itoa3, vec_itoa4, (vector unsigned char)v_out);
			v5 = vec_andc(v5, v_mask1);
			(vector unsigned char)v_out = vec_or(v4, v5);
			
			(vector bool char)v_mask1 = vec_cmplt((vector unsigned char)v_out2, v_32);
			v4 = vec_perm(vec_itoa1, vec_itoa2, (vector unsigned char)v_out2);
			v4 = vec_and(v4, v_mask1);
			v5 = vec_perm(vec_itoa3, vec_itoa4, (vector unsigned char)v_out2);
			v5 = vec_andc(v5, v_mask1);
			(vector unsigned char)v_out2 = vec_or(v4, v5);
			
			(vector bool char)v_mask1 = vec_cmplt((vector unsigned char)v_out3, v_32);
			v4 = vec_perm(vec_itoa1, vec_itoa2, (vector unsigned char)v_out3);
			v4 = vec_and(v4, v_mask1);
			v5 = vec_perm(vec_itoa3, vec_itoa4, (vector unsigned char)v_out3);
			v5 = vec_andc(v5, v_mask1);
			(vector unsigned char)v_out3 = vec_or(v4, v5);
			(vector unsigned char)v_out3 = vec_and(v_zmask, (vector unsigned char)v_out3);
			
			v0 = vec_perm(v_out,v_out2,v_perm1);
			v0 = vec_perm(v0,v_out3,v_perm2);
			v1 = vec_perm(v_out,v_out2,v_perm3);
			v1 = vec_perm(v1,v_out3,v_perm4);
			v2 = vec_perm(v_out,v_out2,v_perm5);
			v2 = vec_perm(v2,v_out3,v_perm6);
			v3 = vec_perm(v_out,v_out2,v_perm7);
			v3 = vec_perm(v3,v_out3,v_perm8);
			
			vec_st(v0, 0, (unsigned int *)rawtrip1);
			ret = method.gettrip(rawtrip1);
			if(method.match(re,rawtrip1,ret,minLength)) {
				memcpy(key, DES_bs_all.keys[i], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
			
			vec_st(v1, 0, (unsigned int *)rawtrip1);
			ret = method.gettrip(rawtrip1);
			if(method.match(re,rawtrip1,ret,minLength)) {
				memcpy(key, DES_bs_all.keys[i+32], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
			
			vec_st(v2, 0, (unsigned int *)rawtrip1);
			ret = method.gettrip(rawtrip1);
			if(method.match(re,rawtrip1,ret,minLength)) {
				memcpy(key, DES_bs_all.keys[i+64], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
			
			vec_st(v3, 0, (unsigned int *)rawtrip1);
			ret = method.gettrip(rawtrip1);
			if(method.match(re,rawtrip1,ret,minLength)) {
				memcpy(key, DES_bs_all.keys[i+96], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", rawtrip1, key);
					fflush(fp);
				}
				[delegate tripFound: rawtrip1 key:key];
			}
		}
		
		counter++;
		gettimeofday(&tv_curr,NULL);
		if((i = tv_curr.tv_sec-tv_prev.tv_sec) >= 5)
		{
			[delegate setCurrentSpeed:(double)(counter*128)/((i*1000000+tv_curr.tv_usec-tv_prev.tv_usec)/1000000.0)];
			tv_prev = tv_curr;
			next = next - counter;
			if(next <= 0) {
				*(key+1) = validsalt[genrand_int32()%64];
				*(key+2) = validsalt[genrand_int32()%64];
				john_crypt_setsalt(key+1);
				next = 100000;
				generate_mode = 93;
			}
			counter = 0;
		}
	}
	
	if(re) free(re);
	if(pe) free(pe);
	if(shouldLog) fclose(fp);
	[pool release];
}

- (void)search4
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	//unsigned int __attribute__ ((aligned (16))) out[8];
	char key[9];
	char trip[12];
	char pattern_str[12];
	char mask_str[12];
	char buf[100];
	char validsalt[64] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	int counter=0, next=100000;
	struct timeval tv_prev, tv_curr;
	register int i;
	//long tmp;
	int j,pattern_num=0, bit,generate_mode=93;
	char *pTrip = trip+1;
	FILE *fp;
	vector unsigned int v_out, v_out2, v_pat_high, v_pat_low, v_mask_high, v_mask_low, v_mask,v_rot;
	vector unsigned int v_result0, v_result1, v_result2, v_resulta, v_resultb, v_resultc, v_resultd;//, v_resulte, v_resultf, v_resultg, v_resulth;
	vector unsigned int v0, v1, v2, v3, v4, v5, v6, v7;//, v8, v9, va, vb, vc, vd, ve, vf;
	register vector unsigned int v_zero;
	register vector unsigned int v_one;
	register vector unsigned int v_max;
	//register vector unsigned char v_perm1 = (vector unsigned char)(0, 1, 2, 3, 8, 9, 10, 11, 16, 17, 18, 19, 24, 25, 26, 27);
	//register vector unsigned char v_perm2 = (vector unsigned char)(4, 5, 6, 7, 12, 13, 14, 15, 20, 21, 22, 23, 28, 29, 30, 31);
	
	trip[0] = -1;
	trip[11] = 0;
	pattern_str[11] = 0;
	mask_str[11] = 0;
	pattern_str[0] = '.';
	mask_str[0] = '?';
	key[8] = 0;
	v_zero = vec_splat_u32(0);
	v_one = vec_splat_u32(1);
	v_max = vec_splat_u32(0xffffffff);
	
	fp = fopen(patternPath, "rb");
	if(!fp) {
		[delegate error: nil type:2];
		[pool release];
		return;
	}
	j = 0;
	while(fgets(buf,100,fp)) j++;
	rewind(fp);
	//NSLog(@"%d",j);
	i = j%4;
	if(i==0) i=4;
	j = j + 4 - i;
	pat_arr = malloc(j*2*sizeof(ARCH_WORD));
	msk_arr = malloc(j*2*sizeof(ARCH_WORD));
	j=0;
	while(fgets(buf,100,fp)) {
		i = strlen(buf);
		if(i == 0) continue;
		if(buf[i-1] == '\n') i--;
		if(i == 0) continue;
		for(;i<10;i++) buf[i] = '?';
		buf[10] = 0;
		strcpy(mask_str+1,buf);
		for(i=1;i<11;i++) {
			if(mask_str[i] == '?') pattern_str[i] = '.';
			else pattern_str[i] = mask_str[i];
		}
		//NSLog(@"%s,%s,%d",pattern_str,mask_str,j);
		generate_pattern2(pattern_str,pat_arr+j);
		generate_mask2(mask_str,msk_arr+j);
		pattern_num++;
		if(j%4 == 3) j += 4;
		j++;
	}
	fclose(fp);
	
	if(pattern_num == 0) {
		[delegate error: nil type:3];
		free(msk_arr);
		free(pat_arr);
		[pool release];
		return;
	}
	
	//NSLog(@"%d,%d",pattern_num,j);
	i = pattern_num % 4;
	if(i == 0) i = 4;
	for(;i<4;i++) {
		//NSLog(@"%s,%s,%d",pattern_str,mask_str,j);
		generate_pattern2(pattern_str,pat_arr+j);
		generate_mask2(mask_str,msk_arr+j);
		pattern_num++;
		if(j%4 == 3) j += 4;
		j++;
	}
	//NSLog(@"%d,%d",pattern_num,j);
	
	if(shouldLog) {
		fp = fopen(logPath, "ab");
		if(!fp) {
			[delegate error: nil type:1];
			free(msk_arr);
			free(pat_arr);
			[pool release];
			return;
		}
	}
	/*
	pat_arr = malloc(2048*sizeof(ARCH_WORD));
	msk_arr = malloc(2048*sizeof(ARCH_WORD));
	for(i=0;i<2048;i++) {
		generate_pattern2(".aaaa......",pat_arr+i);
		generate_mask2("?aaaa??????",msk_arr+i);
		if(i%4 == 3) i += 4;
	}
	pattern_num = 4;
	*/
	*(key+1) = validsalt[genrand_int32()%64];
	*(key+2) = validsalt[genrand_int32()%64];
	john_crypt_setsalt(key+1);
	
	//stime = time(NULL);
	
	gettimeofday(&tv_prev, NULL);
	
	while(running) {
		if(generate_mode < 93) {
			for(i=0;i<128;i++) {
				memcpy(key,DES_bs_all.keys[i],8);
				key[0] = (key[0] < 126) ? key[0]+1 : 33;
				//key[0] = nextchar[key[0]];
				/*key[3] = key[3] < 126 ? key[3]+1 : 33;
				key[4] = key[4] < 126 ? key[4]+1 : 33;
				key[5] = key[5] < 126 ? key[5]+1 : 33;
				key[6] = key[6] < 126 ? key[6]+1 : 33;
				key[7] = key[7] < 126 ? key[7]+1 : 33;*/
				DES_bs_set_key(key, i);
			}
			generate_mode++;
		}
		/*else if(generate_mode < 180) {
			for(i=0;i<128;i++) {
				memcpy(key,DES_bs_all.keys[i],8);
				key[3] = key[3] < 126 ? key[3]+1 : 33;
				DES_bs_set_key(key, i);
			}
			generate_mode++;
		}*/
		else {
			for(i=0;i<128;i++) {
				key[0] = genrand_int32()%94+33;
				key[3] = genrand_int32()%94+33;
				key[4] = genrand_int32()%94+33;
				key[5] = genrand_int32()%94+33;
				key[6] = genrand_int32()%94+33;
				key[7] = genrand_int32()%94+33;
				DES_bs_set_key(key, i);
			}
			generate_mode = 0;
		}
		
		DES_bs_expand_keys();
		DES_bs_crypt_25();
		
		v_mask = vec_splat_u32(1);
		v_rot = (vector unsigned int)(32);
		
		for(i=0;i<32;i++) { /* 4 trips are evaluated per 1 roop */
			//v3 = v_zero; //shamt
			//v3 = v_one;
			v3 = v_rot;
			v_out = v_zero;
			v_out2 = v_zero;
			
			for(bit=0;bit<32;bit++) {
				v1 = vec_ld(0, (unsigned int*)DES_bs_all.B[bit]);
				v2 = vec_ld(0, (unsigned int*)DES_bs_all.B[bit+32]);
				
				v1 = vec_and(v_mask,v1);
				v2 = vec_and(v_mask,v2);
				
				v1 = vec_rl(v1,v3);
				v2 = vec_rl(v2,v3);
				
				v_out = vec_sel(v_out,v_max,v1);
				v_out2 = vec_sel(v_out2,v_max,v2);
				
				v3 = vec_add(v3,v_one);
			}
			
			v_mask = vec_sl(v_mask,v_one);
			v_rot = vec_sub(v_rot,v_one);
			
			v0 = vec_splat(v_out, 0);
			v2 = vec_splat(v_out, 1);
			v_resulta = v_zero;
			v4 = vec_splat(v_out, 2);
			v6 = vec_splat(v_out, 3);
			v_resultb = v_zero;
			v1 = vec_splat(v_out2, 0);
			v3 = vec_splat(v_out2, 1);
			v_resultc = v_zero;
			v5 = vec_splat(v_out2, 2);
			v7 = vec_splat(v_out2, 3);
			v_resultd = v_zero;
			
			for(j=0;j<pattern_num;j+=4) {
				v_pat_high = vec_ld(0, (unsigned int *)(pat_arr+j*2));
				v_pat_low = vec_ld(16, (unsigned int *)(pat_arr+j*2));
				v_mask_high = vec_ld(0, (unsigned int *)(msk_arr+j*2));
				v_mask_low = vec_ld(16, (unsigned int *)(msk_arr+j*2));
				
				v_result0 = vec_and(v0, v_mask_high);
				v_result1 = vec_and(v1, v_mask_low);
				(vector bool int)v_result0 = vec_cmpeq(v_result0, v_pat_high);
				(vector bool int)v_result1 = vec_cmpeq(v_result1, v_pat_low);
				v_result2 = vec_and(v_result0, v_result1);
				v_resulta = vec_or(v_result2, v_resulta);
				
				v_result0 = vec_and(v2, v_mask_high);
				v_result1 = vec_and(v3, v_mask_low);
				(vector bool int)v_result0 = vec_cmpeq(v_result0, v_pat_high);
				(vector bool int)v_result1 = vec_cmpeq(v_result1, v_pat_low);
				v_result2 = vec_and(v_result0, v_result1);
				v_resultb = vec_or(v_result2, v_resultb);
				
				v_result0 = vec_and(v4, v_mask_high);
				v_result1 = vec_and(v5, v_mask_low);
				(vector bool int)v_result0 = vec_cmpeq(v_result0, v_pat_high);
				(vector bool int)v_result1 = vec_cmpeq(v_result1, v_pat_low);
				v_result2 = vec_and(v_result0, v_result1);
				v_resultc = vec_or(v_result2, v_resultc);
				
				v_result0 = vec_and(v6, v_mask_high);
				v_result1 = vec_and(v7, v_mask_low);
				(vector bool int)v_result0 = vec_cmpeq(v_result0, v_pat_high);
				(vector bool int)v_result1 = vec_cmpeq(v_result1, v_pat_low);
				v_result2 = vec_and(v_result0, v_result1);
				v_resultd = vec_or(v_result2, v_resultd);
				
				//continue;
			}
			if(vec_any_eq(v_resulta,v_max)) {
				john_crypt_gettrip4(pTrip,i,masktable[i]);
				memcpy(key, DES_bs_all.keys[i], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", pTrip, key);
					fflush(fp);
				}
				[delegate tripFound: pTrip key:key];
			}
			if(vec_any_eq(v_resultb,v_max)) {
				john_crypt_gettrip4(pTrip,i+32,masktable[i+32]);
				memcpy(key, DES_bs_all.keys[i+32], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", pTrip, key);
					fflush(fp);
				}
				[delegate tripFound: pTrip key:key];
			}
			if(vec_any_eq(v_resultc,v_max)) {
				john_crypt_gettrip4(pTrip,i+64,masktable[i+64]);
				memcpy(key, DES_bs_all.keys[i+64], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", pTrip, key);
					fflush(fp);
				}
				[delegate tripFound: pTrip key:key];
			}
			if(vec_any_eq(v_resultd,v_max))  {
				john_crypt_gettrip4(pTrip,i+96,masktable[i+96]);
				memcpy(key, DES_bs_all.keys[i+96], 8);
				for(j=0;j<8;j++) if(key[j] == 0x22 || key[j] == 0x3c || key[j] == 0x3e) key[j] |= 0x80;
				if(shouldLog) {
					fprintf(fp, "%s : #%s\n", pTrip, key);
					fflush(fp);
				}
				[delegate tripFound: pTrip key:key];
			}
		}
		
		counter++;
		gettimeofday(&tv_curr,NULL);
		if((i = tv_curr.tv_sec-tv_prev.tv_sec) >= 5)
		{
			[delegate setCurrentSpeed:(double)(counter*128)/((i*1000000+tv_curr.tv_usec-tv_prev.tv_usec)/1000000.0)];
			tv_prev = tv_curr;
			next = next - counter;
			if(next <= 0) {
				*(key+1) = validsalt[genrand_int32()%64];
				*(key+2) = validsalt[genrand_int32()%64];
				john_crypt_setsalt(key+1);
				next = 100000;
				generate_mode = 93;
			}
			counter = 0;
		}
		//DES_bs_clear_keys();
	}
	
	if(shouldLog) fclose(fp);
	free(msk_arr);
	free(pat_arr);
	[pool release];
}

- (id)initWithDelegate: (id)del
{
	[super init];
	delegate = [del retain];
	init_genrand(time(NULL));
	DES_bs_init(0);
	generate_shifttable();
	generate_masktable();
	//generate_ofstable();
	strcpy(patternPath, [[[[[NSBundle mainBundle] bundlePath] stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"pattern.txt"] UTF8String]);
	return self;
}

- (void)beginSearch:(NSString *)str mode:(int)m ignoreCase:(BOOL)flag minLength:(int)len log:(BOOL)flag2 study:(BOOL)flag3
{
	searchStr = [str retain];
	mode = m;
	ignoreCase = flag;
	running = YES;
	minLength = len;
	shouldLog = flag2;
	study = flag3;
	if(shouldLog) strcpy(logPath, [[[delegate logPath] stringByAppendingPathComponent:@"VecTripper_out.txt"] UTF8String]);
	if(mode == 0) [NSThread detachNewThreadSelector:@selector(search1) toTarget:self withObject:nil];
	else [NSThread detachNewThreadSelector:@selector(search2) toTarget:self withObject:nil];
}

- (void)beginPatternSearch:(BOOL)flag method:(int)m
{
	searchStr = nil;
	running = YES;
	shouldLog = flag;
	if(shouldLog) strcpy(logPath, [[[delegate logPath] stringByAppendingPathComponent:@"VecTripper_out.txt"] UTF8String]);
	[NSThread detachNewThreadSelector:@selector(search4) toTarget:self withObject:nil];
	//else [NSThread detachNewThreadSelector:@selector(search3) toTarget:self withObject:nil];
}

- (void)stopSearch
{
	running = NO;
	if(searchStr) [searchStr release];
}

@end

